﻿using System;
using System.Collections.Generic;
using System.Linq;
using RootPro.RootProCAD;
using RootPro.RootProCAD.Command;
using RootPro.RootProCAD.Geometry;
using RootPro.RootProCAD.UI;

namespace RCAddInSelectInUndo
{
    [System.AddIn.AddIn("AppAddIn", Version = "1.0", Publisher = "", Description = "")]
    public partial class AppAddIn
    {
        // UNDO/REDOコマンド中に追加/変更された図形
        List<Shape> undoShapeList = new List<Shape>();

        private void AppAddIn_Startup(object sender, EventArgs e)
        {
            Command undoCommand = CommandManager.Commands["RootPro.Edit.Undo"];
            if(undoCommand != null)
            {
                undoCommand.Begin += new CommandBeginEventHandler(command_Begin);
                undoCommand.End += new EventHandler(command_End);
            }
            Command redoCommand = CommandManager.Commands["RootPro.Edit.Redo"];
            if (redoCommand != null)
            {
                redoCommand.Begin += new CommandBeginEventHandler(command_Begin);
                redoCommand.End += new EventHandler(command_End);
            }

            ShapeAdded += new ShapeEventHandler(app_ShapeAdded);
            ShapeChanged += new ShapeEventHandler(app_ShapeChanged);
            ShapeDelete += new ShapeEventHandler(app_ShapeDelete);
        }

        private void AppAddIn_Shutdown(object sender, EventArgs e)
        {
            Command undoCommand = CommandManager.Commands["RootPro.Edit.Undo"];
            if (undoCommand != null)
            {
                undoCommand.Begin -= new CommandBeginEventHandler(command_Begin);
                undoCommand.End -= new EventHandler(command_End);
            }
            Command redoCommand = CommandManager.Commands["RootPro.Edit.Redo"];
            if (redoCommand != null)
            {
                redoCommand.Begin -= new CommandBeginEventHandler(command_Begin);
                redoCommand.End -= new EventHandler(command_End);
            }

            ShapeAdded -= new ShapeEventHandler(app_ShapeAdded);
            ShapeChanged -= new ShapeEventHandler(app_ShapeChanged);
            ShapeDelete -= new ShapeEventHandler(app_ShapeDelete);
        }

        #region VSTA generated code
        private void InternalStartup()
        {
            this.Startup += new System.EventHandler(AppAddIn_Startup);
            this.Shutdown += new System.EventHandler(AppAddIn_Shutdown);
        }
        #endregion

        private void command_Begin(Object sender, EventArgs e)
        {
            undoShapeList.Clear();
        }

        private void command_End(Object sender, EventArgs e)
        {
            // UNDO/REDOコマンド中に追加/変更された図形がある場合
            var command = CommandManager.CurrentCommand;
            if (command != null && (command.UniqueName == "RootPro.Edit.Undo" || command.UniqueName == "RootPro.Edit.Redo"))
            {
                if(undoShapeList.Count > 0)
                {
                    Shape[] shapes = undoShapeList.ToArray();
                    undoShapeList.Clear();

                    try
                    {
                        // 追加/変更された図形を選択状態にする
                        ActiveDocument.SelectionManager.SelectShapes(shapes, SelectionCombineMode.Add);

                        TabView tabView = ActiveTabView;
                        if (tabView != null && tabView is DrawingView)
                        {
                            DrawingView drawView = (DrawingView)tabView;
                            Point2d bottomLeft = drawView.ViewBottomLeftPoint;
                            Point2d topRight = drawView.ViewTopRightPoint;

                            // 追加/変更された図形が表示されるように、画面を縮小する
                            foreach (Shape shape in shapes)
                            {
                                Layout layout = drawView.Drawing.Layouts.First(v => v.Drawing == shape.Drawing);
                                if (layout != null)
                                {
                                    Shape worldShape = drawView.TransformShapeLocalToWorld(layout, shape);
                                    Point2d[] points = worldShape.GetBoundingBox();

                                    if (points[0].X < bottomLeft.X)
                                        bottomLeft.X = points[0].X;
                                    if (points[0].Y < bottomLeft.Y)
                                        bottomLeft.Y = points[0].Y;
                                    if (points[1].X > topRight.X)
                                        topRight.X = points[1].X;
                                    if (points[1].Y > topRight.Y)
                                        topRight.Y = points[1].Y;
                                }
                            }

                            drawView.ZoomWindow(bottomLeft, topRight);
                        }
                    }
                    catch
                    {
                    }
                }
            }
        }

        void app_ShapeAdded(object sender, ShapeEventArgs e)
        {
            // UNDO/REDOコマンド中に追加された図形を保存しておく
            var command = CommandManager.CurrentCommand;
            if (command != null && (command.UniqueName == "RootPro.Edit.Undo" || command.UniqueName == "RootPro.Edit.Redo"))
                undoShapeList.Add(e.Shape);
        }

        void app_ShapeChanged(object sender, ShapeEventArgs e)
        {
            // UNDO/REDOコマンド中に変更された図形を保存しておく
            var command = CommandManager.CurrentCommand;
            if (command != null && (command.UniqueName == "RootPro.Edit.Undo" || command.UniqueName == "RootPro.Edit.Redo"))
                undoShapeList.Add(e.Shape);
        }

        void app_ShapeDelete(object sender, ShapeEventArgs e)
        {
            // UNDO/REDOコマンド中に削除された図形は、リストからも削除しておく
            var command = CommandManager.CurrentCommand;
            if (command != null && (command.UniqueName == "RootPro.Edit.Undo" || command.UniqueName == "RootPro.Edit.Redo"))
            {
                undoShapeList.RemoveAll(v => v == e.Shape);
            }
        }
    }
}
